/*
 * Program: SSL-WOPR Server
 * Source : main.cpp
 * Purpose: Holding main() function and instanciating main WOPR_server object.
 *
 * Copyright (c) 2009, 2010 Oliver Mueller.
 * All rights reserved.
 * http://www.cogito-ergo-sum.org
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the author be held liable for any damages arising from
 * the use of this software.
 *
 */

#include <iostream>
#include <iomanip>

#include "woprexcept.h"
#include "wopr.h"

using namespace std;

int main(int argc, char *argv[])
{
    WOPR_server *server;

    if(argc != 4) {
        cerr << "Usage: wopr host:port certificate.pem private.key" << endl;
        return 1;
    }

    try {
        cout << "WOPR Server" << endl
                << "Listening on " << argv[1] << endl
                << "Using certificate in " << argv[2] << endl
                << "Using private key in " << argv[3] << endl
                << endl;
        server = new WOPR_server(argv[1], argv[2], argv[3]);
        cout << "Initialized" << endl << endl;
        server->run();
        delete server;
    } catch(SSL_exception& se) {
        cerr << "Error: " << se.what() << endl;
        return 1;
    } catch(SSL_init_mutex_exception& sime) {
        cerr << "Error: " << sime.what() << endl;
        return 2;
    } catch(bad_alloc& ba) {
        cerr << "Error: " << ba.what() << endl;
        return 3;
    }

    return 0;
}
