/*
 * Program: SSL-WOPR Server
 * Source : ssl-therad.h
 * Purpose: Declaration of OpenSSL thread callbacks.
 *
 * Copyright (c) 2009, 2010 Oliver Mueller.
 * All rights reserved.
 * http://www.cogito-ergo-sum.org
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the author be held liable for any damages arising from
 * the use of this software.
 *
 */

#ifndef _SSL_THREAD_H
#define	_SSL_THREAD_H

#include <pthread.h>
#include <openssl/crypto.h>

#if OPENSSL_VERSION_NUMBER < 0x00090502f
#error "You need at least OpenSSL 0.9.5b for building this program."
#endif

struct CRYPTO_dynlock_value {
    pthread_mutex_t mutex;
};

#ifdef __IBMCPP__
#if __IBMCPP__ >= 20000
#define PTHREADS_ON_ZOS
#endif
#endif

#ifdef PTHREADS_ON_ZOS
#define ZOS_C_DECL  extern "C"
#else
#define ZOS_C_DECL
#endif

ZOS_C_DECL
struct CRYPTO_dynlock_value *ssl_dynlock_create(const char *file, int line);

ZOS_C_DECL
void ssl_dynlock_destroy(struct CRYPTO_dynlock_value *l, const char *file, int line);

ZOS_C_DECL
void ssl_dynlock_lock(int mode, struct CRYPTO_dynlock_value *l, const char *file, int line);

int ssl_init_mutex();

ZOS_C_DECL
void ssl_lock(int mode, int n, const char *file, int line);

#if OPENSSL_VERSION_NUMBER >= 0x01000000f
// We are using OpenSSL 1.0.0 or higher.
// We will use the new CRYPTO_THREADID_* functions.

ZOS_C_DECL
void ssl_thread_id(CRYPTO_THREADID *id);

#else
// We are using OpenSSL before 1.0.0.
// We have to use the old thread_id based functions
// which are using values of type unsigned long to
// identify threads.

ZOS_C_DECL
unsigned long ssl_thread_id();

#endif /* End of OpenSSL >= 1.0.0? */

#endif	/* _SSL_THREAD_H */

