/*
 * Program: SSL-WOPR Server
 * Source : wopr.h
 * Purpose: Declaration of WOPR_server class.
 *
 * Copyright (c) 2009, 2010 Oliver Mueller.
 * All rights reserved.
 * http://www.cogito-ergo-sum.org
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the author be held liable for any damages arising from
 * the use of this software.
 *
 */

#ifndef __WOPR_H
#define __WOPR_H

#include <iostream>
#include <iomanip>
#include <string>
#include <pthread.h>

#include <openssl/bio.h>
#include <openssl/ssl.h>

#ifdef __IBMCPP__
#if __IBMCPP__ >= 20000
#define C_ON_ZOS
#endif
#endif

using namespace std;

#define STA_LOGSET    0x01
#define STA_LOGUNS    0x02
#define STA_LGNOLY    0x04
#define STA_LGFOLY    0x08
#define STA_SHTDWN    0x10

typedef struct
{
    const char *cmd;
    unsigned cmd_len;
    const char *resp;
    unsigned resp_len;
    short status;
} CmdNResp;

class SSL_exception;
class SSL_init_mutex_exception;
class Overflow_exception;
class WOPR_server;

class WOPR_client_attrib
{
public:
    WOPR_client_attrib(WOPR_server *server, unsigned long client_number, SSL *ssl, BIO *client);
    ~WOPR_client_attrib();

    unsigned long get_number();
    SSL *get_ssl();
    BIO *get_client();
    WOPR_server *get_server();
    
protected:
    unsigned long client_number;
    SSL *ssl;
    BIO *client;
    WOPR_server *server;
};

inline unsigned long WOPR_client_attrib::get_number()
{
    return client_number;
}

inline SSL *WOPR_client_attrib::get_ssl()
{
    return ssl;
}

inline BIO *WOPR_client_attrib::get_client()
{
    return client;
}

inline WOPR_server *WOPR_client_attrib::get_server()
{
    return server;
}

class WOPR_server
{
public:
    WOPR_server(char *host, const char *cert, const char *key) throw (SSL_exception, SSL_init_mutex_exception, bad_alloc);
    ~WOPR_server();
    void run() throw (SSL_exception);

#ifdef C_ON_ZOS
    // dirty trick for z/OS
    void handle_client(WOPR_client_attrib *wca);
#endif

protected:
    SSL_CTX *ctx;
    BIO *abio;
    unsigned long clno;
    char errmsgbuf[ERRSIZE];

    static pthread_mutex_t cout_mutex;
    static pthread_mutex_t cerr_mutex;

    static bool ssl_initialized;
    static bool wopr_initialized;

    static const char *identification;
    static const char *prompt[2];
    static unsigned prompt_len[2];
    static const char *unknown_cmd;
    static unsigned unknown_cmd_len;
    static const char *unknown_user;
    static unsigned unknown_user_len;
    static CmdNResp cmd_n_resp[];

    static const char *access_denied;
    static const char *already_running;

#ifndef C_ON_ZOS
    void handle_client(WOPR_client_attrib *wca);
#endif
    void print_error(unsigned long n, string s);
    void print_info(unsigned long n, string s);
    void print_info2(unsigned long n, string s1, string s2);
    void trim_all(char *s);

#ifndef C_ON_ZOS
    static void* _thread(void* attrib);
#endif
};

#ifdef C_ON_ZOS
// dirty trick for z/OS
extern "C" void* _thread(void* attrib);
#endif

#endif
